@@ -1,6 +1,6 @@
 // vi:ts=4:shiftwidth=4:expandtab
 /***************************************************************************
-                          global.h  -  global definition
+                          debug.h  -  debug routine
                              -------------------
     begin                : Sun Mar 18 2001
     copyright            : (C) 2001 by ejoy
@@ -16,7 +16,7 @@
  *                                                                         *
  ***************************************************************************/
 #ifndef DEBUG_H
-#define  DEBUG_H
+#define DEBUG_H
 
 #ifndef NDEBUG
 #include <fstream>
