@@ -22,6 +22,7 @@
 using namespace std;
 
 #include <unistd.h>
+#include <sys/time.h>
 #include <string>
 
 #include "global.h"
@@ -34,17 +35,26 @@ using namespace std;
 #include "inputclient.h"
 #include "nativeinputserver.h"
 #ifdef HAVE_UNICON_LIB
-	#include "uniconinputserver.h"
+    #include "uniconinputserver.h"
 #endif
 #include "configserver.h"
 #include "nativebarclient.h"
 #include "overspotclient.h"
 #include "inputmanager.h"
 
-InputManager::InputManager( Zhcon * pOwner, Console * pCon, InputStyle style,
-    string& OverSpotColors, string& NativeBarColors)
-        : mpOwner(pOwner),
-        mpCon(pCon),
+int InputManager::mConFd = -1;
+int InputManager::mConNo = -1;
+int InputManager::mTtyFd = -1;
+
+void InputManager::SetTty(int confd, int ttyno, int ttyfd) {
+    mConFd = confd;
+    mConNo = ttyno;
+    mTtyFd = ttyfd;
+}
+
+InputManager::InputManager( Console * pCon,
+    InputStyle style, string& OverSpotColors, string& NativeBarColors)
+        : mpCon(pCon),
         mpInputServer(NULL),
         mpInputClient(NULL),
         mActive(false),
@@ -54,13 +64,16 @@ InputManager::InputManager( Zhcon * pOwner, Console * pCon, InputStyle style,
         msOverSpotColors(OverSpotColors),
         msNativeBarColors(NativeBarColors),
         mpHelpWin(NULL) {
-    mClientFd = pOwner->GetClientFd();
-    if (mStyle == NativeBar)
-        mpInputClient = new NativeBarClient(msNativeBarColors);
-    else if (mStyle == OverSpot)
+
+    assert(mTtyFd >= 0);
+    if (mConNo >= 0) mMouse.Open(mpCon, mConFd, mConNo, mTtyFd);
+    
+    InputClient::SetConsole(pCon);
+    if (mStyle == OverSpot)
         mpInputClient = new OverSpotClient(msOverSpotColors);
-    else
-        throw runtime_error("unknown input style.");
+    else // if (mStyle == NativeBar)
+        mpInputClient = new NativeBarClient(msNativeBarColors);
+        
     KDSysSet();
     Active();
     // set VT size
@@ -68,6 +81,7 @@ InputManager::InputManager( Zhcon * pOwner, Console * pCon, InputStyle style,
 }
 
 InputManager::~InputManager() {
+    mMouse.Close();
     //HelpHide(); may cause core dump if mpConsole is deleted first
     delete mpHelpWin;
     delete mpInputServer;
@@ -76,7 +90,74 @@ InputManager::~InputManager() {
     KDInputRestore();
 }
 
-void InputManager::ProcessKey(char c) {
+void InputManager::Process(InputEvt &evt) {
+    evt.oper = InputEvt::Nothing;
+    evt.sub = 0;
+    
+    const int BUFSIZE = 8192;
+    char buf[BUFSIZE];
+    struct timeval tv;
+    int count, rcved, shift;
+
+    shift = GetShiftState();
+    if (shift != 0) {
+        //printf("Shift stat %d\r\n", s);
+    }
+
+    // must use SetTty after construct
+    assert(mConFd >= 0 && mTtyFd >= 0);
+
+    FD_ZERO(&mFdSet);
+    FD_SET(mConFd, &mFdSet);
+    FD_SET(mTtyFd, &mFdSet);
+#ifdef HAVE_GPM_H  
+    if (mMouse.IsOpen()) FD_SET(mMouse.mFd, &mFdSet);
+#endif 
+               
+    tv.tv_sec = 0;
+    tv.tv_usec = 100000;                  /* 0.1 sec */
+    rcved = select(FD_SETSIZE, &mFdSet, NULL, NULL, &tv);
+    if (rcved <= 0)
+        return;
+
+#ifdef HAVE_GPM_H
+    if (mMouse.IsOpen()) {
+        if (FD_ISSET(mMouse.mFd, &mFdSet)) {
+            mMouse.Process();
+        }
+    }
+#endif
+
+   if (FD_ISSET(mConFd, &mFdSet)) {
+        count = read(mConFd, buf, BUFSIZE);
+        if (count > 0) {
+            for (int i = 0; i < count; i++)
+               ProcessKey(buf[i], evt);
+        }
+   }
+
+   if (FD_ISSET(mTtyFd, &mFdSet)) {
+        count = read(mTtyFd, buf, BUFSIZE);
+        if (count > 0) {
+            mpCon->Write(buf, count);
+        }
+    }
+}
+
+int InputManager::GetShiftState()
+{
+   int shift = 0;
+#if defined(linux)
+   int arg = 6;
+   if (ioctl(mConFd, TIOCLINUX, &arg) != -1)
+   {
+       shift = arg;
+   }
+#endif
+   return shift;
+}
+
+void InputManager::ProcessKey(char c, InputEvt &evt) {
     if (c == ALT_SPACE) { // always useful
         if (mActive)
             DisActive();
@@ -84,7 +165,7 @@ void InputManager::ProcessKey(char c) {
             Active();
         return ;
     }
-    if (ProcessSysKey(c)) // is sys keyb
+    if (ProcessSysKey(c, evt)) // is sys keyb
         return ;
 
     HelpHide();
@@ -97,8 +178,13 @@ void InputManager::ProcessKey(char c) {
         return ;
 
     if (mpInputClient->Visible() && mpInputServer) {
+        bool isContinue;
         string s;
-        mpInputServer->ProcessKey(c, s);
+        isContinue = mpInputServer->ProcessKey(c, s);
+        if (!isContinue) { // speical for ConfigServer
+            DoCtrlSpace();
+            return;
+        }
         if (!s.empty())
             WriteClient(s);
         mpInputClient->Update();
@@ -109,24 +195,28 @@ void InputManager::ProcessKey(char c) {
 
 //return true if a sys-key is processed
 //return false:no sys-key found!
-
-bool InputManager::ProcessSysKey(char c) {
+bool InputManager::ProcessSysKey(char c, InputEvt &evt) {
     bool r = true;
     switch (c) {
         case CTRL_F1:
-            mpOwner->SetEncode(GB2312, GB2312);
+            evt.oper = InputEvt::SetEnco;
+            evt.sub = 1;
             break;
         case CTRL_F2:
-            mpOwner->SetEncode(GBK, GBK);
+            evt.oper = InputEvt::SetEnco;
+            evt.sub = 2;
             break;
         case CTRL_F3:
-            mpOwner->SetEncode(BIG5, BIG5);
+            evt.oper = InputEvt::SetEnco;
+            evt.sub = 3;
             break;
         case CTRL_F4:
-            mpOwner->SetEncode(JIS, JIS);
+            evt.oper = InputEvt::SetEnco;
+            evt.sub = 4;
             break;
         case CTRL_F5:
-            mpOwner->SetEncode(KSCM, KSCM);
+            evt.oper = InputEvt::SetEnco;
+            evt.sub = 5;
             break;
         case CTRL_F7:
             switch (mStyle) {
@@ -143,22 +233,8 @@ bool InputManager::ProcessSysKey(char c) {
 
             break;
         case CTRL_F9:
-            switch (mpOwner->mAutoEncode) {
-                case Zhcon::AUTO:
-                    mpOwner->mAutoEncode = Zhcon::AUTO_GB;
-                    break;
-                case Zhcon::AUTO_GB:
-                    mpOwner->mAutoEncode = Zhcon::AUTO_BIG5;
-                    break;
-                case Zhcon::AUTO_BIG5:
-                    mpOwner->mAutoEncode = Zhcon::MANUAL;
-                    break;
-                case Zhcon::MANUAL:
-                    mpOwner->mAutoEncode = Zhcon::AUTO;
-                    break;
-            }
-            Redraw();
-            Beep();
+            evt.oper = InputEvt::AutoEncoSwitch;
+            evt.sub = 0;
             break;
         case CTRL_F10:
             MenuMode();
@@ -238,12 +314,12 @@ bool InputManager::ProcessInputKey(char c) {
     return r;
 }
 
-inline void InputManager::WriteClient(char c) {
-    write(mClientFd, &c, 1);
+void InputManager::WriteClient(char c) {
+    write(mTtyFd, &c, 1);
 }
 
-inline void InputManager::WriteClient(string & s) {
-    write(mClientFd, s.c_str(), s.size());
+void InputManager::WriteClient(string & s) {
+    write(mTtyFd, s.c_str(), s.size());
 }
 
 void InputManager::Active() {
@@ -276,8 +352,8 @@ void InputManager::SetVtSize() {
     struct winsize ws;
     ws.ws_col = cols;
     ws.ws_row = rows;
-//    debug<<"Change VT size "<<cols<<","<<rows<<endl;
-    ioctl(mClientFd, TIOCSWINSZ, &ws);
+    // debug<<"Change VT size "<<cols<<","<<rows<<endl;
+    ioctl(mTtyFd, TIOCSWINSZ, &ws);
 }
 
 //circle through available input methods
@@ -475,7 +551,6 @@ InputManager::KeyMap InputManager::mKDInputMap[] = {
     {(1 << KG_CTRL) + (1 << KG_ALT), 9, 0, K(KT_LATIN, CTRL_ALT_8)},
     {(1 << KG_CTRL) + (1 << KG_ALT), 10, 0, K(KT_LATIN, CTRL_ALT_9)},
     //ALT_X
-    /*
     {1 << KG_ALT, 11, 0, K(KT_LATIN, ALT_0)},
     {1 << KG_ALT, 2, 0, K(KT_LATIN, ALT_1)},
     {1 << KG_ALT, 3, 0, K(KT_LATIN, ALT_2)},
@@ -486,7 +561,6 @@ InputManager::KeyMap InputManager::mKDInputMap[] = {
     {1 << KG_ALT, 8, 0, K(KT_LATIN, ALT_7)},
     {1 << KG_ALT, 9, 0, K(KT_LATIN, ALT_8)},
     {1 << KG_ALT, 10, 0, K(KT_LATIN, ALT_9)},
-    */
     //CTRL_X
     {1 << KG_CTRL, 42, 0, K(KT_LATIN, CTRL_SHIFT)},
     {1 << KG_CTRL, 57, 0, K(KT_LATIN, CTRL_SPACE)},
@@ -551,7 +625,8 @@ void InputManager::KDInputRestore() {
 
 #elif defined(__FreeBSD__)
 #include <termios.h>
-#include <machine/console.h>
+//#include <machine/console.h>
+#include <sys/consio.h>
 
 keymap_t InputManager::mKDSysOld;
 keymap_t InputManager::mKDSysNew;
@@ -613,7 +688,6 @@ void InputManager::KDInputSet() {
         mKDInputNew.key[11].map[6] = CTRL_ALT_0;
 
         // ALT_X
-        /*
         mKDInputNew.key[2].map[4] = ALT_1;
         mKDInputNew.key[3].map[4] = ALT_2;
         mKDInputNew.key[4].map[4] = ALT_3;
@@ -624,7 +698,6 @@ void InputManager::KDInputSet() {
         mKDInputNew.key[9].map[4] = ALT_8;
         mKDInputNew.key[10].map[4] = ALT_9;
         mKDInputNew.key[11].map[4] = ALT_0;
-        */
 
         // CTRL_X
         mKDInputNew.key[42].map[2] = CTRL_SHIFT;
@@ -665,7 +738,7 @@ void InputManager::ChangeStyle(InputStyle style) {
     // set VT size
     SetVtSize();
     //force api fresh screen
-    write(mClientFd, "\0", 1);
+    write(mTtyFd, "\0", 1);
 
     if (visible) {
         mpInputClient->Update();
@@ -689,9 +762,9 @@ void InputManager::HelpShow() {
         return ;
 
     int c1, r1, c2, r2;
-    c1 = (gpConsole->MaxCols() - 60) / 2;
+    c1 = (mpCon->MaxCols() - 60) / 2;
     c2 = c1 + 61;
-    r1 = (gpConsole->MaxRows() - 18) / 2;
+    r1 = (mpCon->MaxRows() - 18) / 2;
     r2 = r1 + 18;
     mpHelpWin = new Window(c1 * gpScreen->BlockWidth(),
                            r1 * gpScreen->BlockHeight(),
@@ -703,34 +776,34 @@ void InputManager::HelpShow() {
     mpHelpWin->SetBgColor(4);
     mpHelpWin->Clear();
 
-    mpHelpWin->PutStr(16, 0, _("Press CTRL-ALT-H to exit help"));
-    mpHelpWin->PutStr(1, 1, _("ALT -SPACE  Open/Close CJK mode"));
-    mpHelpWin->PutStr(1, 2, _("CTRL-SPACE  Open/Close input method"));
-    mpHelpWin->PutStr(1, 3, _("SHIFT-PageUp/PageDown/Up/Down   Scroll & Display History"));
+    mpHelpWin->PutStr(16, 0,  _("Press CTRL-ALT-H to exit help"));
+    mpHelpWin->PutStr(1, 1,   _("ALT -SPACE  Open/Close CJK mode"));
+    mpHelpWin->PutStr(1, 2,   _("CTRL-SPACE  Open/Close input method"));
+    mpHelpWin->PutStr(1, 3,   _("SHIFT-PageUp/PageDown/Up/Down   Scroll & Display History"));
 
-    mpHelpWin->PutStr(1, 5, _("CTRL key:"));
-    mpHelpWin->PutStr(1, 6, _("F1  GB2312"));
-    mpHelpWin->PutStr(15, 6, _("F2  GBK"));
-    mpHelpWin->PutStr(30, 6, _("F3  BIG5"));
-    mpHelpWin->PutStr(45, 6, _("F4  JIS"));
-    mpHelpWin->PutStr(1, 7, _("F5  KSCM"));
-    mpHelpWin->PutStr(15, 7, _("F9  Switch & Auto Detect"));
-    mpHelpWin->PutStr(45, 7, _("F10 Sys Menu"));
-    mpHelpWin->PutStr(1, 8, _("F7  Input Style"));
-    mpHelpWin->PutStr(45, 8, _("D   Exit zhcon"));
+    mpHelpWin->PutStr(1, 5,   _("CTRL key:"));
+    mpHelpWin->PutStr(1, 6,   _("F1  GB2312"));
+    mpHelpWin->PutStr(15, 6,  _("F2  GBK"));
+    mpHelpWin->PutStr(30, 6,  _("F3  BIG5"));
+    mpHelpWin->PutStr(45, 6,  _("F4  JIS"));
+    mpHelpWin->PutStr(1, 7,   _("F5  KSC"));
+    mpHelpWin->PutStr(15, 7,  _("F9  Switch & Auto Detect"));
+    mpHelpWin->PutStr(45, 7,  _("F10 Sys Menu"));
+    mpHelpWin->PutStr(1, 8,   _("F7  Input Style"));
+    mpHelpWin->PutStr(45, 8,  _("D   Exit zhcon"));
 
-    mpHelpWin->PutStr(1, 10, _("Chinese/English Mode:"));
-    mpHelpWin->PutStr(1, 11, _("CTRL-,  Full/Half Char"));
-    mpHelpWin->PutStr(1, 12, _("CTRL-.  Full/Half Comma"));
+    mpHelpWin->PutStr(1, 10,  _("Chinese/English Mode:"));
+    mpHelpWin->PutStr(1, 11,  _("CTRL-,  Full/Half Char"));
+    mpHelpWin->PutStr(1, 12,  _("CTRL-.  Full/Half Comma"));
 
     mpHelpWin->PutStr(30, 10, _("CTRL-ALT-NUM:"));
     mpHelpWin->PutStr(30, 11, _("0      Prompt Mode"));
     mpHelpWin->PutStr(30, 12, _("1..9   Select IME 1..9"));
 
-    mpHelpWin->PutStr(1, 14, _("Special Input Key:"));
-    mpHelpWin->PutStr(1, 15, _("CTRL-SHIFT  Next IME"));
+    mpHelpWin->PutStr(1, 14,  _("Special Input Key:"));
+    mpHelpWin->PutStr(1, 15,  _("CTRL-SHIFT  Next IME"));
     mpHelpWin->PutStr(30, 15, _("SPACE       Select Number 0"));
-    mpHelpWin->PutStr(1, 16, _("+/-         Next/Prev page"));
+    mpHelpWin->PutStr(1, 16,  _("+/-         Next/Prev page"));
     mpHelpWin->PutStr(30, 16, _("ESC         Reset Input Area"));
 
     mpHelpWin->Show();
@@ -744,3 +817,5 @@ void InputManager::HelpHide() {
     delete mpHelpWin;
     mpHelpWin = NULL;
 }
+
+
