@@ -21,10 +21,9 @@
 
 #include <iostream>
 #include <strstream>
-#include <sys/time.h>
 #include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
+#include <cstdlib>
+#include <cstdio>
 #include <sys/ioctl.h>
 
 #if defined(linux)
@@ -34,7 +33,8 @@
     #include <pty.h>
 #elif defined(__FreeBSD__)
     #include <termios.h>
-    #include <machine/console.h>
+    //#include <machine/console.h>
+    #include <sys/consio.h>
     #include <libutil.h>
     #define TCSETA TIOCSETA
 #endif
@@ -47,6 +47,7 @@
 #include <stdexcept>
 #include <algorithm>
 #include <locale.h>
+
 #include "global.h"
 #include "console.h"
 
@@ -57,13 +58,16 @@
     #include "uniconinputserver.h"
 #endif
 
+// for termcap, must include after gpm.h, why?
+#include <term.h>
+
 #include "gbkdecoder.h"
 #include "gbdecoder.h"
 #include "big5decoder.h"
 #include "gb2big5decoder.h"
 #include "big52gbdecoder.h"
 #include "jisdecoder.h"
-#include "kscmdecoder.h"
+#include "kscdecoder.h"
 #include "configfile.h"
 //#include "fade.h"
 #include "zhcon.h"
@@ -77,13 +81,13 @@ BaseFont* gpAscFont;
 ofstream debug("debug");
 #endif
 
-int gfd;
+Zhcon* Zhcon::mpZhcon = NULL;
 int Zhcon::mTtyPid = 0;
 Zhcon::STATE Zhcon::mState = STOP;
 
 char* Zhcon::mpTips[] = {
 gettext_noop("Please visit zhcon.gnuchina.org for more information."),
-gettext_noop("Zhcon supports GB2312,GBK,BIG5,JIS,KSCM encode."),
+gettext_noop("Zhcon supports GB2312,GBK,BIG5,JIS,KSC encode."),
 gettext_noop("You can use CTRL-ALT-0 to toggle prompt bar."),
 gettext_noop("Press CTRL-ALT-H for online help."),
 gettext_noop("Zhcon supports 12,14,16,24 pixel fonts,you can change in zhcon.conf"),
@@ -92,20 +96,33 @@ gettext_noop("Hit CTRL-D to quit zhcon."),
 gettext_noop("You can find user manual in doc/."),
 gettext_noop("Zhcon can use input methods from Unicon[TurboLinux],read manual for detail."),
 gettext_noop("Zhcon is optimized for running under Linux FrameBuffer."),
+gettext_noop("Start from version 0.2.1,zhcon support gpm mouse in console."),
 gettext_noop("Press CTRL-F7 to change input style")
 };
 
-Zhcon::Zhcon()
+void Zhcon::SignalVtLeave(int signo) {
+    if(Zhcon::mpZhcon != NULL) mpZhcon->VtDoLeave();
+}
+
+void Zhcon::SignalVtEnter(int signo) {
+    if(Zhcon::mpZhcon != NULL) mpZhcon->VtDoEnter();
+}
+
+Zhcon::Zhcon(int argc, char* argv[])
 :mpCon(NULL),
 mpInputManager(NULL),
-mpBar(NULL),
-mShowTips(true)
+mShowTips(true),
+mArgc(argc),
+mArgv(argv)
 {
+    Zhcon::mpZhcon = this;
+    mConFd = mTtyFd = -1;
     random_shuffle(mpTips,mpTips + sizeof(mpTips)/sizeof(char*) );
 }
 
 Zhcon::~Zhcon() {
     CleanUp();
+    Zhcon::mpZhcon = NULL;
 }
 
 void Zhcon::Init() {
@@ -115,7 +132,8 @@ void Zhcon::Init() {
     if (access(cfgfile.c_str(), R_OK) != 0)
         cfgfile = "/etc/zhcon.conf";
 
-//    char c;cin>>c;
+    //for debug,a pause enable us to attach zhcon's pid in gdb
+    //char c;cin>>c;
     ConfigFile f(cfgfile.c_str());
     //the InitXXX sequence is important,do not change
     //unless you know what you are doing
@@ -126,71 +144,103 @@ void Zhcon::Init() {
     InitLocale(f); // include init font
     InitCon(f);
     InitMisc(f);
-    InstallVtHandle();
+    VtSignalLeave();
     InstallSignal();
     ForkPty();
+    // if setlocale run before forkpty, it cause FreeBSD hang
+    SetEncode(mDefaultEncode, mDefaultEncode);
     InitInputManager(f);
     if (f.GetOption("startupmsg",true))
         StartupMsg();
 }
 
 void Zhcon::Run() {
-    fd_set set;
-    char buf[BUFSIZE];
-    struct timeval tv;
-    int n,r,t;
+    InputEvt evt;
+    Encode e;
+
+    int t = 0;
 //    fadeout();
 //    fadein();
     mState = RUNNING;
     while (mState == RUNNING) {
-        t = 0;
-        Encode e;
-        do {
-            mpCon->CursorBlink();
-            FD_ZERO(&set);
-            FD_SET(mConFd, &set);
-            FD_SET(mTtyFd, &set);
-            tv.tv_sec = 0;
-            tv.tv_usec = 100000;                  /* 0.1 sec */
-            r = select(FD_SETSIZE, &set, NULL, NULL, &tv);
-            //auto detect buffer encode after idle for 1 sec
-            if (++t == 10 && mAutoEncode != MANUAL) {
-                e = mpCon->DetectBufferEncode();
-                switch (e) {
-                    case ASCII:
-                        SetEncode(mDefaultEncode, mDefaultEncode);
-                        break;
-                    case GB2312:
-                        SetEncode(GB2312,
-                                  mAutoEncode ==
-                                  AUTO_BIG5 ? BIG5 : GB2312); break;
-                    case BIG5:
-                        SetEncode(BIG5,
-                                  mAutoEncode ==
-                                  AUTO_GB ? GB2312 : BIG5); break;
-                    default:
-                        assert(!"Wrong encode!");
-                }
-            }
-        } while (r <= 0);
-        //read or write
-
-        if (FD_ISSET(mConFd, &set)) {
-            n = read(mConFd, buf, BUFSIZE);
-            if (n > 0)
-                for (int i = 0; i < n; i++)
-                    mpInputManager->ProcessKey(buf[i]);
+        mpCon->CursorBlink();
+        mpInputManager->Process(evt);
+        if (evt.oper != InputEvt::Nothing) {
+            DoInputEvt(evt);
+            continue;
         }
 
-        if (FD_ISSET(mTtyFd, &set)) {
-            n = read(mTtyFd, buf, BUFSIZE);
-            if (n > 0) {
-                mpCon->Write(buf, n);
-            }
+        if (++t < 10 || mAutoEncode == MANUAL)
+            continue;
+            
+        t = 0;
+        e = mpCon->DetectBufferEncode();
+        switch (e) {
+            case ASCII:
+                SetEncode(mDefaultEncode, mDefaultEncode);
+                break;
+            case GB2312:
+                SetEncode(GB2312,
+                          mAutoEncode ==
+                          AUTO_BIG5 ? BIG5 : GB2312);
+                break;
+            case BIG5:
+                SetEncode(BIG5,
+                          mAutoEncode ==
+                          AUTO_GB ? GB2312 : BIG5);
+                break;
+            default:
+               assert(!"Wrong encode!");
         }
     } //while
 }
 
+void Zhcon::DoInputEvt(InputEvt &evt) {
+    switch (evt.oper) {
+        case InputEvt::SetEnco:
+            switch (evt.sub) {
+                case 1:
+                    SetEncode(GB2312, GB2312);
+                    break;
+                case 2:
+                    SetEncode(GBK, GBK);
+                    break;
+                case 3:
+                    SetEncode(BIG5, BIG5);
+                    break;
+                case 4:
+                    SetEncode(JIS, JIS);
+                    break;
+                case 5:
+                    SetEncode(KSC, KSC);
+                    break;
+                default:
+                    break;
+            }
+            break;
+        case InputEvt::AutoEncoSwitch:
+            switch (mAutoEncode) {
+                case AUTO:
+                    mAutoEncode = AUTO_GB;
+                    break;
+                case AUTO_GB:
+                    mAutoEncode = AUTO_BIG5;
+                    break;
+                case AUTO_BIG5:
+                    mAutoEncode = MANUAL;
+                    break;
+                case MANUAL:
+                    mAutoEncode = AUTO;
+                    break;
+            }
+            mpInputManager->Redraw();
+            Beep();
+            break;
+        default:
+            break;
+    }
+}
+
 //fork a new pty to run user's shell
 //return true if successfully fork
 void Zhcon::ForkPty() {
@@ -216,7 +266,11 @@ void Zhcon::ForkPty() {
             close(i);
         setsid();
         seteuid(getuid());   /* for security */
-        execlp(userpd->pw_shell, userpd->pw_shell, (char *) 0);
+        //TODO: handle error condition
+        if (mArgc > 1)
+            execvp(mArgv[1], &mArgv[1]);
+        else
+            execlp(userpd->pw_shell, userpd->pw_shell, (char *) 0);
         exit(-1);
     }
 }
@@ -263,14 +317,7 @@ void Zhcon::CleanUp() {
     sigaction(SIGBUS, &act, NULL);
     sigaction(SIGFPE, &act, NULL);
 
-    struct vt_mode vtm;
-    signal(SIGUSR1, SIG_DFL);
-    signal(SIGUSR2, SIG_DFL);
-    vtm.mode = VT_AUTO;
-    vtm.waitv = 0;
-    vtm.relsig = 0;
-    vtm.acqsig = 0;
-    ioctl(0, VT_SETMODE, &vtm);
+    VtSignalClean();
 #if defined(__FreeBSD__)
     ioctl(0, VT_RELDISP, 1);
 #endif
@@ -281,14 +328,36 @@ void Zhcon::CleanUp() {
     delete gpDecoder;
     delete mpCon;
     delete mpInputManager;
-    //need restore signal ?
-    tcsetattr(mConFd, TCSAFLUSH, &mOldTermios);
-    ioctl(mConFd, TIOCSWINSZ, &mOldWinSize);
-    ioctl(mConFd, KDSETMODE, KD_TEXT);
-    setenv("LC_ALL", mOldLocale.c_str(), 1);
+    if (mConFd != -1) {
+        // need restore signal ?
+        tcsetattr(mConFd, TCSAFLUSH, &mOldTermios);
+        // restore console blanking
+#if defined(linux)
+        write(mConFd, "\033[9;10]",7);  // default 10 minutes
+#elif defined(__FreeBSD__) 
+        int BlankTime = 5;  // default 5 minutes
+        ioctl(mConFd, CONS_BLANKTIME, &BlankTime);
+#endif
+        ioctl(mConFd, KDSETMODE, KD_TEXT);  // special for vga
+        ioctl(mConFd, TIOCSWINSZ, &mOldWinSize);
 
+        // for mouse, keep text mode clean
+        // write(mConFd, "\n", 1);
+
+        // make visible cursor, use termcap "ve" instead
+        //write(mConFd, "\E[?25h\E[?0c", 11);
+        if (mpCapCursorOn) write(mConFd, mpCapCursorOn, strlen(mpCapCursorOn));
+
+        close(mConFd);
+    }
+
+    setenv("LC_ALL", mOldLocale.c_str(), 1);
 }
 
+char  Zhcon::mCapBuf[512] = {0};
+char* Zhcon::mpCapClearScr = NULL;
+char* Zhcon::mpCapCursorOff = NULL;
+char* Zhcon::mpCapCursorOn = NULL;
 void Zhcon::InitTty() {
     // Using throw cause core dump when call destruct
     if (!isatty(fileno(stdout))) {
@@ -335,10 +404,53 @@ void Zhcon::InitTty() {
     t.c_cc[VSUSP] = _POSIX_VDISABLE;
     t.c_cc[VDSUSP] = _POSIX_VDISABLE;
     t.c_cc[VQUIT] = _POSIX_VDISABLE;
+    setenv("TERM","linux",1);
 #endif
     if (tcsetattr(mConFd, TCSAFLUSH, &t) < 0)
         throw runtime_error("Can't set console termios\n");
-    ioctl(0, TIOCGWINSZ, &mOldWinSize);
+    ioctl(mConFd, TIOCGWINSZ, &mOldWinSize);
+    
+    setenv("PTY_SLAVE", TtyName, 1);
+
+    // disable console auto blanking to prevent screen redraw
+    // problem when waking up
+#if defined(linux)
+    // see kernel's console.c, function setterm_command
+    write(mConFd, "\033[9;0]",6);
+#elif defined(__FreeBSD__) 
+    // man splash
+    // see /usr/src/sys/dev/syscons/syscons.c
+    int BlankTime = 0;
+    ioctl(mConFd, CONS_BLANKTIME, &BlankTime);
+#endif
+
+    // for mouse, keep text mode clean
+    char* TermType = getenv("TERM");
+    if (TermType == NULL) {
+        // printf("can't find the TERM environment variable.");
+        return;
+    }
+    int rc = tgetent(NULL, TermType);
+    if (rc == -1) {
+        //printf("can't find the termcap database.\n");
+        //printf("check your termcap environment variable ...\n");
+        return;
+    }
+    if (rc == 0) {
+        //printf("can't find the terminal type in the termcap database.\n");
+        return;
+    }
+    char *pCap = mCapBuf;
+    mpCapClearScr = tgetstr("cl", &pCap);
+    mpCapCursorOff = tgetstr("vi", &pCap);
+    mpCapCursorOn  = tgetstr("ve",  &pCap);
+
+    if (mpCapClearScr) write(mConFd, mpCapClearScr, strlen(mpCapClearScr));
+    if (mpCapCursorOff) write(mConFd, mpCapCursorOff, strlen(mpCapCursorOff));
+    // clear screen and home cursor, use termcap "cl" instead
+    //write(mConFd, "\E[H\E[J", 6);
+    // invisible cursor even in text mode, use termcap "vi" instead
+    //write(mConFd, "\E[?25l\E[?1c", 11);
 }
 
 void Zhcon::SetEncode(Encode e, Encode font) {
@@ -383,10 +495,10 @@ void Zhcon::SetEncode(Encode e, Encode font) {
                 h = new JISDecoder();
                 locale = "ja.JIS";
                 break;
-            case KSCM:
+            case KSC:
                 assert(font == e);
-                f = new BaseFont(mKSCMFont, 16, 16);
-                h = new KSCMDecoder();
+                f = new BaseFont(mKSCFont, 16, 16);
+                h = new KSCDecoder();
                 locale = "ko";
                 break;
             default:
@@ -404,67 +516,86 @@ void Zhcon::SetEncode(Encode e, Encode font) {
         textdomain(PACKAGE);
         //Fix me:how to change child's shell env?
         if (mpCon) {
+            mpCon->SelClear();
             mpCon->ResetFlagAll();
             mpCon->Redraw(true);
         }
-        if (gpInputManager)
-            gpInputManager->Redraw();
+        if (mpInputManager)
+            mpInputManager->Redraw();
         Beep();
     }//try
     catch (runtime_error & e) {}
 }
 
-/*setup vt switch handle */
-void Zhcon::InstallVtHandle() {
+void Zhcon::VtSignalSet(int mode)
+{
+    //return; for debug
     vt_mode vtm;
     if (ioctl(mConFd, VT_GETMODE, &vtm))
         throw runtime_error("ioctl VT_GETMODE failed!");
 
-    vtm.mode = VT_PROCESS;
-    vtm.relsig = SIGUSR1;
-    vtm.acqsig = SIGUSR2;
-#if defined(__FreeBSD__)
-    // based on cce
-    vtm.frsig = SIGUSR1;  //added by raner or ioctl will fail
-#endif
-    signal(SIGUSR1, ::SignalVtLeave);
-    signal(SIGUSR2, ::SignalVtEnter);
+    switch (mode) {
+        case 1:  // leave
+            signal(SIGUSR1, Zhcon::SignalVtLeave);
+            vtm.mode = VT_PROCESS;
+            vtm.relsig = SIGUSR1;
+            vtm.acqsig = SIGUSR1; // for FreeBSD, see syscons.c, 0 < signal < NSIG
+            vtm.frsig = SIGUSR1;
+            break;
+        case 2:  // enter
+            signal(SIGUSR1, Zhcon::SignalVtEnter);
+            vtm.mode = VT_PROCESS;
+            vtm.relsig = SIGUSR1; 
+            vtm.acqsig = SIGUSR1;
+            vtm.frsig = SIGUSR1;
+            break;
+        default:  // clean
+            signal(SIGUSR1, SIG_DFL);
+            vtm.mode = VT_AUTO;
+            break;
+    }
+    
     if (ioctl(mConFd, VT_SETMODE, &vtm))
         throw runtime_error("ioctl VT_SETMODE failed!");
 }
 
-void Zhcon::SignalVtLeave() {
+void Zhcon::VtDoLeave() {
+    mpCon->SelClear();
     mpCon->CursorHide();  // when window has cursor, put in hide()
     mpCon->Hide();
     mpInputManager->Hide();
-    signal(SIGUSR1, ::SignalVtLeave);
+
     // not needed, may change color platte when libggi used
     // TextMode();
     gpScreen->SwitchToText();
     ioctl(mConFd, VT_RELDISP, 1);
+
+    VtSignalEnter();
 }
 
-void Zhcon::SignalVtEnter() {
-    signal(SIGUSR2, ::SignalVtEnter);
+void Zhcon::VtDoEnter() {
     // GraphMode();
     gpScreen->SwitchToGraph();
-    gpScreen->FillRect(0, 0, gpScreen->Width() - 1, gpScreen->Height() - 1,
-                       0);
+    gpScreen->ClearScr();
     mpCon->Show();
     mpCon->CursorShow();  // when window has cursor, put in show()
     mpInputManager->Show();
     ioctl(mConFd, VT_RELDISP, VT_ACKACQ);
+
+    VtSignalLeave();
 }
 
 void Zhcon::GraphMode() {
-    if (ioctl(mConFd, KDSETMODE, KD_GRAPHICS))
-        throw runtime_error("ioctl KDSETMODE failed!");
+    // for mouse, keep text mode
+    //if (ioctl(mConFd, KDSETMODE, KD_GRAPHICS))
+    //    throw runtime_error("ioctl KDSETMODE failed!");
     ioctl(mConFd, TIOCCONS, NULL);
 }
 
 void Zhcon::TextMode() {
-    if (ioctl(mConFd, KDSETMODE, KD_TEXT))
-        throw runtime_error("ioctl KDSETMODE failed!");
+    // for mouse, keep text mode
+    //if (ioctl(mConFd, KDSETMODE, KD_TEXT))
+    //    throw runtime_error("ioctl KDSETMODE failed!");
     ioctl(mConFd, TIOCCONS, NULL);
 }
 
@@ -480,7 +611,7 @@ void Zhcon::InitLocale(ConfigFile& f){
     mGBKFont = prefix + f.GetOption(string("gbkfont"), string(GBKFONT));
     mBIG5Font = prefix + f.GetOption(string("big5font"), string(BIG5FONT));
     mJISFont = prefix + f.GetOption(string("jisfont"), string(JISFONT));
-    mKSCMFont = prefix + f.GetOption(string("kscmfont"), string(KSCMFONT));
+    mKSCFont = prefix + f.GetOption(string("kscfont"), string(KSCFONT));
 
     gpAscFont = new BaseFont(mASCIIFont, 8, 16);
     gpScreen->SetAscFont(gpAscFont);
@@ -489,23 +620,29 @@ void Zhcon::InitLocale(ConfigFile& f){
     string s;
     s = f.GetOption(string("defaultencode"), string("gb2312"));
     if (s == "gb2312") {
-        SetEncode(GB2312,GB2312);
+        //SetEncode(GB2312,GB2312);
+        //FreeBsd use zh_CN.EUC instead of zh_CN.GB2312
+        //a broken locale? workaround it
+#if defined (__FreeBSD__)
+        setenv("LC_ALL", "zh_CN.EUC", 1);
+#else
         setenv("LC_ALL", "zh_CN.GB2312", 1);
+#endif
         mDefaultEncode = GB2312;
     } else if (s == "gbk") {
-        SetEncode(GBK,GBK);
+        //SetEncode(GBK,GBK);
         setenv("LC_ALL", "zh_CN.GBK", 1);
         mDefaultEncode = GBK;
     } else if (s == "big5") {
-        SetEncode(BIG5,BIG5);
+        //SetEncode(BIG5,BIG5);
         setenv("LC_ALL", "zh_TW.Big5", 1);
         mDefaultEncode = BIG5;
     } else if (s == "jis") {
-        SetEncode(JIS,JIS);
+        //SetEncode(JIS,JIS);
         setenv("LC_ALL", "ja.JIS", 1);
         mDefaultEncode = JIS;
-    } else if (s == "kscm") {
-        SetEncode(KSCM,KSCM);
+    } else if (s == "ksc") {
+        //SetEncode(KSC,KSC);
         setenv("LC_ALL", "ko", 1);
         mDefaultEncode = GBK;
     } else {
@@ -541,7 +678,8 @@ void Zhcon::InitGraphDev(ConfigFile& f){
     GraphDev::mBlankLineHeight = f.GetOption(string("blanklineheight"), 0);
     if (GraphDev::mBlankLineHeight < 0)
         GraphDev::mBlankLineHeight = 0;
-    gpScreen->FillRect(0, 0, gpScreen->Width() - 1, gpScreen->Height() - 1,0);
+    // debug << gpScreen->Width() << "," << gpScreen->Height() << endl;
+    gpScreen->ClearScr();
 }
 
 void Zhcon::InitCon(ConfigFile& f){
@@ -559,10 +697,22 @@ void Zhcon::InitCon(ConfigFile& f){
     int CursorType;
     CursorType = f.GetOption(string("cursortype"), CUR_DEF);
     mpCon->SetCursorType(CursorType);
-    gpConsole = mpCon;
+
+    // for mouse support
+    struct winsize ws;
+    ws.ws_col = mpCon->MaxCols();
+    ws.ws_row = mpCon->MaxRows();
+    // debug << "tty size " << mConFd << " " << ws.ws_col << "," << ws.ws_row << endl;
+    ioctl(mConFd, TIOCSWINSZ, &ws);
 }
 
 void Zhcon::InitInputManager(ConfigFile& f){
+    assert(mConFd >= 0); // set by InitTty()
+    char *TtyName = ttyname(mConFd);
+    int ttyno = atoi(&TtyName[8]);  // must be /dev/tty? style
+    assert(mTtyFd >= 0); // set by ForkPty()
+    InputManager::SetTty(mConFd, ttyno, mTtyFd);
+    
     string s;
     s = f.GetOption(string("zhconpath"), string(PREFIX"/lib/zhcon/"));
     NativeInputServer::SetDataPath(s);
@@ -575,37 +725,47 @@ void Zhcon::InitInputManager(ConfigFile& f){
     sNativeBar = f.GetOption(string("nativebarcolor"), string("15,4,11,14,0,12"));
     s = f.GetOption(string("inputstyle"),string("overspot"));
     if (s == "overspot")
-        mpInputManager = new InputManager(this,mpCon,InputManager::OverSpot, sOverSpot, sNativeBar);
-    else if (s == "nativebar")
-        mpInputManager = new InputManager(this,mpCon,InputManager::NativeBar, sOverSpot, sNativeBar);
-    else
-        throw runtime_error("unknown input style.");
+        mpInputManager = new InputManager(mpCon, InputManager::OverSpot, sOverSpot, sNativeBar);
+    else // others as default, "nativebar")
+        mpInputManager = new InputManager(mpCon, InputManager::NativeBar, sOverSpot, sNativeBar);
+
     mpInputManager->LoadImmInfo(f);
     mpInputManager->Show();
-    gpInputManager = mpInputManager;
     if (f.GetOption("promptmode",true))
-        gpInputManager->PromptMode();
+        mpInputManager->PromptMode();
 }
 
 void Zhcon::InitMisc(ConfigFile& f){
     Beep(f.GetOption(string("beep"),true));
 }
+
 void Zhcon::StartupMsg(){
     #include "logo.h"
     string scr = screendata;
-    string s;
-    s = "Linux   ";
+    string s = PLATFORM;
+    s.resize(8,' ');
     scr.replace(scr.find("PLATFORM"),8,s);
     s = VERSION;
     s.resize(7,' ');
     scr.replace(scr.find("VERSION"),7,s);
     strstream buf;
     buf<<_("screen resolution")<<": "<<gpScreen->Width()<<"X"<<
-        gpScreen->Height()<<" ["<<gpConsole->MaxCols()<<","<<
-        gpConsole->MaxRows()<<"]"<<"\r\n";
+        gpScreen->Height()<<" ["<<mpCon->MaxCols()<<","<<
+        mpCon->MaxRows()<<"]"<<"\r\n";
     buf<<_("default encode")<<": "<<GetEncode()<<"\r\n";
-    scr += buf.str();
-    gpConsole->Write(scr.c_str(),scr.size());
+    /*
+    if (mpInputManager->mMouse.mFd >= 0) {
+        buf<<_("Mouse")<<": "<<_("On");
+        if (mpInputManager->mMouse.GetButtons() == 3)
+            buf<<", 3 " << _("buttons");
+        buf<<"\r\n";
+    }
+    else
+        buf<<_("Mouse")<<": "<<_("Off")<<"\r\n";
+    */
+    // no pcount, may display garbage
+    scr += string(buf.str(), buf.pcount());
+    mpCon->Write(scr.c_str(),scr.size());
 }
 
 string Zhcon::GetEncode(){
@@ -640,8 +800,8 @@ string Zhcon::GetEncode(){
         case JIS:
             s += "JIS";
             break;
-        case KSCM:
-            s += "KSCM";
+        case KSC:
+            s += "KSC";
             break;
         default:
             s += "Unknown";
